'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.checkLogin = checkLogin;
exports.searchUser = searchUser;
exports.userList = userList;
exports.clientList = clientList;
exports.sendTotp = sendTotp;
exports.addClient = addClient;
exports.generateSecret = generateSecret;
exports.roleList = roleList;
exports.addRole = addRole;
exports.deleteRole = deleteRole;

var _querystring = require('querystring');

var _querystring2 = _interopRequireDefault(_querystring);

var _util = require('../util');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function* checkLogin(next) {
  if (this.session.user) {
    if (this.session.user.is_admin) {
      this.state.user = this.session.user;
      yield* next;
    } else {
      this.redirect(this._routes.home);
    }
  } else {
    this.session.returnTo = this.url;
    this.redirect(this._routes.login);
  }
}

function* searchUser() {
  const User = this.orm().User;
  const q = this.request.body.q || '';
  const users = yield User.findAll({
    attributes: ['email'],
    where: {
      enable: 1,
      email: {
        $like: q + '%'
      }
    },
    offset: 0,
    limit: 15
  });
  this.body = users.map(u => u.email);
}

function* userList() {
  var _orm = this.orm();

  const User = _orm.User;

  const page = parseInt(this.query.p, 10) || 1;
  const query = this.query.q || '';
  const limit = 20;
  const offset = (page - 1) * limit;
  const where = { enable: 1 };
  if (query) {
    where['email'] = { $like: `%${query}%` };
  }
  const users = yield User.findAndCountAll({
    where,
    limit,
    offset,
    order: 'email ASC',
    attributes: {
      exclude: ['pass_hash', 'pass_salt']
    }
  });

  yield this.render('admin/users', {
    page,
    users,
    query,
    navUsers: 'active',
    total: Math.ceil(users.count / limit),
    link: p => `?${_querystring2.default.stringify({ q: query, p })}`
  });
}

function* clientList() {
  var _orm2 = this.orm();

  const Client = _orm2.Client;

  const page = parseInt(this.query.p, 10) || 1;
  const query = this.query.q || '';
  const limit = 20;
  const offset = (page - 1) * limit;
  const where = {};
  if (query) {
    where['name'] = { $like: `%${query}%` };
  }
  const clients = yield Client.findAndCountAll({
    where,
    limit,
    offset,
    order: 'name ASC',
    attributes: ['id', 'secret', 'redirect_uri', 'name']
  });

  yield this.render('admin/clients', {
    page,
    query,
    clients,
    navClients: 'active',
    total: Math.ceil(clients.count / limit),
    link: p => `?${_querystring2.default.stringify({ q: query, p })}`
  });
}

function* sendTotp() {
  const User = this.orm().User;
  const id = this.request.body.id;


  if (!id) {
    this.flash('error', 'ID is required');
    this.redirect(this._routes.admin.users);
    return;
  }

  try {
    // generate new totp key
    let res = yield User.update({
      totp_key: (0, _util.generateToken)()
    }, {
      where: { id }
    });
    // send email
    if (res[0]) {
      let user = yield User.findById(id);
      yield this.sendMail(user.email, 'send_totp', {
        username: user.email,
        cid: 'key',
        email: user.email,
        key: (0, _util.encodeKey)(user.totp_key)
      }, [{
        filename: 'key.png',
        content: (0, _util.totpImage)(user.email, user.totp_key),
        cid: 'key'
      }]);
    } else {
      this.flash('error', 'Update failed');
      this.redirect(this._routes.admin.users);
      return;
    }
    this.flash('success', 'Reset and send TOTP key successfully');
    this.redirect(this._routes.admin.users);
  } catch (e) {
    console.error(e.stack);
    this.flash('error', 'Reset and send key failed');
    this.redirect(this._routes.admin.users);
  }
}

function* addClient() {
  const Client = this.orm().Client;
  var _request$body = this.request.body;
  const name = _request$body.name,
        redirect_uri = _request$body.redirect_uri;


  if (!name) {
    this.flash('error', 'Name is required');
    this.redirect(this._routes.admin.clients);
    return;
  }

  if (!redirect_uri) {
    this.flash('error', 'Redirect URI is required');
    this.redirect(this._routes.admin.clients);
    return;
  }

  try {
    // add one new
    yield Client.create({
      secret: (0, _util.generateToken)(),
      name: name,
      redirect_uri: redirect_uri
    });

    this.flash('success', 'Add new client successfully');
    this.redirect(this._routes.admin.clients);
  } catch (e) {
    console.error(e.stack);
    this.flash('error', 'Add new client failed');
    this.redirect(this._routes.admin.clients);
  }
}

function* generateSecret() {
  const Client = this.orm().Client;
  const id = this.request.body.id;


  if (!id) {
    this.flash('error', 'ID is required');
    this.redirect(this._routes.admin.clients);
    return;
  }

  try {
    // generate new secret
    let res = yield Client.update({
      secret: (0, _util.generateToken)()
    }, {
      where: {
        id: id
      }
    });
    if (res[0]) {
      this.flash('success', 'Generate new secret successfully');
      this.redirect(this._routes.admin.clients);
    } else {
      this.flash('error', 'Update failed');
      this.redirect(this._routes.admin.clients);
      return;
    }
  } catch (e) {
    console.error(e.stack);
    this.flash('error', 'Generate new secret failed');
    this.redirect(this._routes.admin.clients);
  }
}

function* roleList() {
  var _orm3 = this.orm();

  const User = _orm3.User,
        Role = _orm3.Role,
        Client = _orm3.Client,
        DicRole = _orm3.DicRole;

  const page = parseInt(this.query.p, 10) || 1;
  const query = this.query.q || '';
  const where = {};
  if (query) {
    const temp = yield User.findAll({
      offset: 0,
      limit: 100,
      attributes: ['id'],
      where: {
        email: { $like: `%${query}%` }
      }
    });
    where['user_id'] = {
      $in: temp.map(v => v.id)
    };
  }
  const limit = 20;
  const offset = (page - 1) * limit;
  const roles = yield Role.findAndCountAll({
    where,
    limit,
    offset,
    order: 'user_id ASC',
    attributes: ['id', 'user_id', 'client_id', 'role']
  });
  const users = yield User.findAll({
    attributes: ['id', 'email'],
    where: {
      id: { $in: roles.rows.map(v => v.user_id) }
    }
  });
  const userMap = users.reduce((o, c) => {
    o[c.id] = c.email;
    return o;
  }, {});

  const clients = yield Client.findAll();
  const dics = yield DicRole.findAll();
  const clientMap = clients.reduce((o, c) => {
    o[c.id] = c.name;
    return o;
  }, {});
  yield this.render('admin/roles', {
    dics,
    page,
    query,
    roles,
    clients,
    userMap,
    clientMap,
    navRoles: 'active',
    total: Math.ceil(roles.count / limit),
    link: p => `?${_querystring2.default.stringify({ q: query, p })}`
  });
}

function* addRole() {
  var _orm4 = this.orm();

  const Role = _orm4.Role,
        User = _orm4.User;
  var _request$body2 = this.request.body;
  const email = _request$body2.email,
        client = _request$body2.client,
        role = _request$body2.role;


  if (!email) {
    this.flash('error', 'Email is required');
    this.redirect(this._routes.admin.roles);
    return;
  }

  if (!client) {
    this.flash('error', 'Client is required');
    this.redirect(this._routes.admin.roles);
    return;
  }

  if (!role) {
    this.flash('error', 'Role is required');
    this.redirect(this._routes.admin.roles);
    return;
  }

  const user = yield User.findByEmail(email);
  if (!user) {
    this.flash('error', 'User is not existed');
    this.redirect(this._routes.admin.roles);
    return;
  }

  try {
    // add one new
    yield Role.create({
      user_id: user.id,
      client_id: client,
      role: role
    });

    this.flash('success', 'Add new role successfully');
    this.redirect(this._routes.admin.roles);
  } catch (e) {
    console.error(e.stack);
    this.flash('error', 'Add new role failed, maybe it is existed');
    this.redirect(this._routes.admin.roles);
  }
}

function* deleteRole() {
  const Role = this.orm().Role;
  const id = this.request.body.id;


  if (!id) {
    this.flash('error', 'Id is required');
    this.redirect(this._routes.admin.roles);
    return;
  }
  // Delete role
  const num = yield Role.destroy({
    where: { id }
  });

  if (num <= 0) {
    this.flash('error', 'Delete role failed, maybe it is not existed');
    this.redirect(this._routes.admin.roles);
    return;
  }

  this.flash('success', 'Delete role successfully');
  this.redirect(this._routes.admin.roles);
}