'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.home = home;
exports.checkLogin = checkLogin;
exports.login = login;
exports.session = session;
exports.logout = logout;
exports.passwordResetPage = passwordResetPage;
exports.passwordReset = passwordReset;
exports.passwordChangePage = passwordChangePage;
exports.passwordChange = passwordChange;
exports.getInfo = getInfo;
exports.sendToken = sendToken;

var _isEmail = require('validator/lib/isEmail');

var _isEmail2 = _interopRequireDefault(_isEmail);

var _notp = require('notp');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function* home() {
  if (this.config.redirectURL) {
    this.redirect(this.config.redirectURL);
  } else {
    yield this.render('home', {
      user: this.session.user
    });
  }
}

function* checkLogin(next) {
  if (this.session.user) {
    yield next;
  } else {
    this.session.returnTo = this.url;
    this.redirect(this._routes.login);
  }
}

function* login() {
  yield this.render('login', {
    isTOTP: this.config.isTOTP
  });
}

function* session() {
  var _orm = this.orm();

  const User = _orm.User;
  var _request$body = this.request.body;
  const email = _request$body.email,
        password = _request$body.password,
        token = _request$body.token;


  if (!email) {
    this.flash('error', 'Email is required');
    this.redirect(this._routes.login);
    return;
  }
  if (!password) {
    this.flash('error', 'Password is required');
    this.redirect(this._routes.login);
    return;
  }
  if (this.config.isTOTP && !token) {
    this.flash('error', 'Token is required');
    this.redirect(this._routes.login);
    return;
  }

  const user = yield User.auth(email, password);

  if (!user) {
    this.flash('error', 'Email or password is invalid');
    this.redirect(this._routes.login);
    return;
  }

  if (this.config.isTOTP && !_notp.totp.verify(token, user.totp_key, { window: 3 })) {
    this.flash('error', 'Token is invalid');
    this.redirect(this._routes.login);
    return;
  }

  const returnTo = this.session.returnTo;

  this.session.returnTo = null;
  this.session.user = user;
  this.redirect(returnTo || this._routes.home);
}

function* logout(next) {
  const returnTo = this.query.return_to;
  this.session.user = null;
  this.redirect(returnTo || this._routes.login);
  yield next;
}

function* passwordResetPage() {
  yield this.render('reset');
}

function* passwordReset() {
  var _orm2 = this.orm();

  const User = _orm2.User,
        EmailCode = _orm2.EmailCode;
  const email = this.request.body.email;


  if (!email || !(0, _isEmail2.default)(email)) {
    this.flash('error', 'Email is empty or invalid type');
    this.redirect(this._routes.password_reset);
    return;
  }

  const user = yield User.findByEmail(email);
  if (!user) {
    this.flash('error', 'User is not found');
    this.redirect(this._routes.password_reset);
    return;
  }

  try {
    const code = yield EmailCode.create({
      user_id: user.id
    });
    yield this.sendMail(user.email, 'password_reset', {
      username: user.email,
      ttl: this.config.emailCodeTTL / 3600,
      link: this.config.domain + this._routes.password_change + '?code=' + code.id
    });
    this.flash('success', 'Check your email for a link to reset your password.');
    this.redirect(this._routes.login);
  } catch (e) {
    console.error(e.stack);
    this.flash('error', 'Send email failed');
    this.redirect(this._routes.password_reset);
  }
}

function* passwordChangePage() {
  var _orm3 = this.orm();

  const EmailCode = _orm3.EmailCode;


  let codeId = this.query.code;

  if (!codeId) {
    this.flash('error', 'Code is required');
    this.redirect(this._routes.password_reset);
    return;
  }

  let code = yield EmailCode.findById(codeId);
  if (!code) {
    this.flash('error', 'Code is invalid');
    this.redirect(this._routes.password_reset);
    return;
  }

  let expiresAt = code.createdAt.getTime() + this.config.emailCodeTTL * 1000;
  if (expiresAt < Date.now()) {
    this.flash('error', 'Code is expired');
    this.redirect(this._routes.password_reset);
    return;
  }
  yield this.render('change', {
    codeId: codeId,
    title: 'Change password'
  });
}

function* passwordChange() {
  var _orm4 = this.orm();

  const User = _orm4.User,
        EmailCode = _orm4.EmailCode;
  var _request$body2 = this.request.body;
  const password = _request$body2.password,
        password2 = _request$body2.password2,
        codeId = _request$body2.codeId;


  if (!codeId) {
    this.flash('error', 'Code is required');
    this.redirect(this._routes.password_reset);
    return;
  }

  const code = yield EmailCode.findById(codeId);
  if (!code) {
    this.flash('error', 'Code is invalid');
    this.redirect(this._routes.password_reset);
    return;
  }

  const expiresAt = code.createdAt.getTime() + this.config.emailCodeTTL * 1000;
  if (expiresAt < Date.now()) {
    this.flash('error', 'Code is expired');
    this.redirect(this._routes.password_reset);
    return;
  }

  if (!password || !password2 || password !== password2 || password.length < 8) {
    this.flash('error', 'Password is invalid');
    this.redirect('back');
    return;
  }

  // TODO: add transaction
  yield User.changePassword(code.user_id, password);
  yield EmailCode.destroy({
    where: {
      id: codeId
    }
  });

  this.flash('success', 'Password have changed');
  this.redirect(this._routes.login);
}

function* getInfo(next) {
  var _orm5 = this.orm();

  const User = _orm5.User;


  this.body = yield User.findById(this._userId, {
    attributes: ['id', 'email'],
    raw: true
  });
}

function* sendToken() {
  var _orm6 = this.orm();

  const User = _orm6.User;
  const email = this.request.body.email;
  const lastTime = this.session.lastTime;


  this.assert(email, 400, 'Email is required');
  // wait 1 minute
  const min = 60 * 1000;
  const now = Date.now();
  this.assert(!lastTime || now - lastTime > min, 400, 'Try again in a minute');
  this.session.lastTime = now;

  const user = yield User.findByEmail(email);
  this.assert(user, 400, 'User is not found');
  yield this.sendMail(email, 'send_token', {
    username: email,
    sender: this.config.mail.from,
    token: _notp.totp.gen(user.totp_key, { time: this.config.dynamicTokenTTL })
  });

  this.body = { code: 0 };
}